@@ -4,11 +4,22 @@ require 'httpclient'
 require 'totoridipjp'
 
 module Plugin::PhotoSupport
+  SUPPORTED_IMAGE_FORMATS = GdkPixbuf::Pixbuf.formats.flat_map{|f| f.extensions }.freeze
   INSTAGRAM_PATTERN = %r{\Ahttps?://(?:instagr\.am|(?:www\.)?instagram\.com)/p/([a-zA-Z0-9_\-]+)/}
+  GITHUB_IMAGE_PATTERN = %r<\Ahttps://github\.com/(\w+/\w+)/blob/(.*\.(?:#{SUPPORTED_IMAGE_FORMATS.join('|')}))\z>
 
   class << self
     extend Memoist
 
+    def via_xpath(display_url, xpath)
+      connection = HTTPClient.new
+      page = connection.get_content(display_url)
+      unless page.empty?
+        doc = Nokogiri::HTML(page)
+        doc.xpath(xpath).first
+      end
+    end
+
     # Twitter cardsのURLを画像のURLに置き換える。
     # HTMLを頻繁にリクエストしないように、このメソッドを通すことでメモ化している。
     # ==== Args
@@ -16,12 +27,21 @@ module Plugin::PhotoSupport
     # ==== Return
     # String 画像URL(http://d250g2.com/d250g2.jpg)
     def d250g2(display_url)
-      connection = HTTPClient.new
-      page = connection.get_content(display_url)
-      unless page.empty?
-        doc = Nokogiri::HTML(page)
-        doc.css('meta[name="twitter:image:src"]').first.attribute('content') end end
+      r = via_xpath(display_url, "//meta[@name='twitter:image']/@content")
+      return r if r
+      via_xpath(display_url, "//meta[@name='twitter:image:src']/@content")
+    end
     memoize :d250g2
+
+    # OpenGraphProtocol対応のURLを画像のURLに置き換える。
+    # HTMLを頻繁にリクエストしないように、このメソッドを通すことでメモ化している。
+    # ==== Args
+    # [display_url] https://www.instagram.com/p/Bj4XIgNHacT/
+    # ==== Return
+    # String 画像URL(https://scontent-nrt1-1.cdninstagram.com/vp/867c287aac67e555f873458042d25c70/5BC16C10/t51.2885-15/e35/33958857_788214964721700_3554146954256580608_n.jpg)
+    def インスタ映え(display_url)
+      via_xpath(display_url, "//meta[@property='og:image']/@content") end
+    memoize :"インスタ映え"
   end
 end
 
@@ -39,16 +59,6 @@ Plugin.create :photo_support do
     open(result.attribute('src'))
   end
 
-  # twipple photo
-  defimageopener('twipple photo', %r<^http://p\.twipple\.jp/[a-zA-Z0-9]+>) do |display_url|
-    connection = HTTPClient.new
-    page = connection.get_content(display_url)
-    next nil if page.empty?
-    doc = Nokogiri::HTML(page)
-    result = doc.css('#post_image').first
-    open(result.attribute('src'))
-  end
-
   # moby picture
   defimageopener('moby picture', %r<^http://moby.to/[a-zA-Z0-9]+>) do |display_url|
     connection = HTTPClient.new
@@ -61,12 +71,8 @@ Plugin.create :photo_support do
 
   # gyazo
   defimageopener('gyazo', %r<\Ahttps?://gyazo.com/[a-zA-Z0-9]+>) do |display_url|
-    connection = HTTPClient.new
-    page = connection.get_content(display_url)
-    next nil if page.empty?
-    doc = Nokogiri::HTML(page)
-    result = doc.css('.image').first
-    open(result.attribute('src'))
+    img = Plugin::PhotoSupport.d250g2(display_url)
+    open(img) if img
   end
 
   # 携帯百景
@@ -103,11 +109,6 @@ Plugin.create :photo_support do
     open(result.attribute('src'))
   end
 
-  # twitgoo
-  defimageopener('twitgoo', %r<^http://twitgoo\.com/[a-zA-Z0-9]+>) do |display_url|
-    open(display_url)
-  end
-
   # jigokuno.com
   defimageopener('jigokuno.com', %r<^http://jigokuno\.com/\?eid=\d+>) do |display_url|
     connection = HTTPClient.new
@@ -138,15 +139,9 @@ Plugin.create :photo_support do
   end
 
   # Fotolog
-  defimageopener('Fotolog', %r<http://(?:www\.)fotolog\.com/\w+/\d+/?>) do |display_url|
-    connection = HTTPClient.new
-    page = connection.get_content(display_url)
-    next nil if page.empty?
-    doc = Nokogiri::HTML(page)
-    result = doc.css('meta').lazy.find_all{ |dom|
-      'og:image' == dom.attribute('property').to_s
-    }.first
-    open(result.attribute('content'))
+  defimageopener('Fotolog', %r<\Ahttps?://(?:www\.)?fotolog\.com/\w+/\d+/?>) do |display_url|
+    img = Plugin::PhotoSupport.インスタ映え(display_url)
+    open(img) if img
   end
 
   # フォト蔵
@@ -160,13 +155,8 @@ Plugin.create :photo_support do
 
   # instagram
   defimageopener('instagram', Plugin::PhotoSupport::INSTAGRAM_PATTERN) do |display_url|
-    notice display_url
-    connection = HTTPClient.new
-    page = connection.get_content(display_url)
-    next nil if page.empty?
-    doc = Nokogiri::HTML(page)
-    result = doc.xpath("//meta[@property='og:image']/@content").first
-    open(result)
+    img = Plugin::PhotoSupport.インスタ映え(display_url)
+    open(img) if img
   end
 
   # d250g2
@@ -214,16 +204,6 @@ Plugin.create :photo_support do
     open(result.attribute('content').value)
   end
 
-  defimageopener('彩の庭', %r{\Ahttp://haruicon\.com/ayanoniwa?\Z}) do |display_url|
-    connection = HTTPClient.new
-    page = connection.get_content(display_url)
-    next nil if page.empty?
-    doc = Nokogiri::HTML(page)
-    path = doc.css('img').attribute('src').value
-    img = URI.join("http://haruicon.com", path)
-    open(img)
-  end
-
   # xkcd.com
   defimageopener('xkcd', %r<\Ahttps?://xkcd\.com/[0-9]+>) do |display_url|
     connection = HTTPClient.new
@@ -237,4 +217,65 @@ Plugin.create :photo_support do
     end
     open(src)
   end
+
+  # マシュマロ
+  defimageopener('marshmallow-qa', %r<\Ahttps?://marshmallow-qa\.com/messages/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}>) do |display_url|
+    img = Plugin::PhotoSupport.インスタ映え(display_url)
+    open(img) if img
+  end
+
+  # peing
+  defimageopener('peing', %r<\Ahttps?://peing\.net/\w+/(?:qs/\d+|q/[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12})>) do |display_url|
+    img = Plugin::PhotoSupport.d250g2(display_url)
+    open(img) if img
+  end
+
+  # ニコニコ静画
+  defimageopener('NicoSeiga', %r<\Ahttps?://seiga\.nicovideo\.jp/seiga/im\d+>) do |display_url|
+    img = Plugin::PhotoSupport.インスタ映え(display_url)
+    open(img) if img
+  end
+
+  # ニコニコ静画 (nico.ms経由)
+  defimageopener('NicoSeiga via nico.ms', %r<\Ahttps?://nico\.ms/im\d+>) do |display_url|
+    connection = HTTPClient.new
+    location = connection.get(display_url).header['Location'].first
+    next nil if location.nil?
+    img = Plugin::PhotoSupport.インスタ映え(location)
+    open(img) if img
+  end
+
+  # GitHub
+  defimageopener('github', Plugin::PhotoSupport::GITHUB_IMAGE_PATTERN) do |display_url|
+    url = Plugin::PhotoSupport::GITHUB_IMAGE_PATTERN.match(display_url) do |m|
+      "https://raw.githubusercontent.com/#{m[1]}/#{m[2]}"
+    end
+    open(url) if url
+  end
+
+  # Amazon
+  amazon_json_regex1 = %r!'colorImages': { 'initial': (\[.*MAIN.*\])!
+  amazon_json_regex2 = %r! data-a-dynamic-image="([^"]*)"!
+  defimageopener('Amazon', %r<\Ahttps?://(?:www\.amazon\.(?:co\.jp|com)/(?:.*/)?(?:dp/|gp/product/)|amzn.asia/d/)[0-9A-Za-z_]+>) do |url|
+    html = HTTPClient.new.get_content(url, [], [['User-Agent', 'mikutter']])
+    m = amazon_json_regex1.match(html)
+    via_data_attr = false
+    if m.nil?
+      m = amazon_json_regex2.match(html)
+      via_data_attr = true
+    end
+    next nil unless m
+    json = m[1]
+    unless via_data_attr
+      arr = JSON.parse(json, symbolize_names: true)
+      main = arr.find {|v| v[:variant] == "MAIN" }
+      image_url = main[:large]
+    else
+      json = CGI.unescapeHTML(json)
+      hash = JSON.parse(json)
+      max = hash.to_a.max {|v| v[1][0] }
+      image_url = max[0]
+    end
+    open(image_url)
+  end
 end
